package com.halo.halojuhesdk.core.base.utils;

import android.database.Cursor;
import android.os.Environment;
import android.text.TextUtils;


import com.halo.halojuhesdk.core.common.utils.MyUtils;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * 
 * @ClassName: FileUtils
 * 
 * @Description: 文件读写工具类
 * 
 * @author Jeven QQ:2723402941
 * 
 * @date 2015-6-12 下午1:48:03
 * 
 */
public class FileUtils {
	public static final String DOWNLOAD_DIR = "download";
	public static final String CACHE_DIR = "cache";
	public static final String ICON_DIR = "icon";

	/**
	 * 判断SD卡是否挂载
	 */
	public static boolean isSDCardAvailable() {
		return Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState());
	}

	/**
	 * 获取下载目录
	 */
	public static String getDownloadDir() {
		return getDir(isSDCardAvailable(), DOWNLOAD_DIR);
	}

	/**
	 * 获取缓存目录
	 */
	public static String getCacheDir() {
		return getDir(isSDCardAvailable(), CACHE_DIR);
	}

	/**
	 * 获取icon目录
	 */
	public static String getIconDir() {
		return getDir(isSDCardAvailable(), ICON_DIR);
	}

	/**
	 * 获取应用目录，当SD卡存在时，获取SD卡上的目录，当SD卡不存在时，获取应用的cache目录
	 */
	public static String getDir(boolean isSDCardAvailable, String name) {
		try {
			StringBuilder sb = new StringBuilder();
			if (isSDCardAvailable) {
				sb.append(getExternalStoragePath());
			} else {
				sb.append(getCachePath());
			}
			sb.append(name);
			sb.append(File.separator);
			String path = sb.toString();
			if (createDirs(path)) {
				LogUtils.d("   创建SD卡路径  :  " + path);
				return path;
			} else if (isSDCardAvailable) {
				LogUtils.d("   在SD上创建目录失败" + path);
				getDir(false, name);
			}
		} catch (Exception e) {
			LogUtils.e("   获取SD卡路径失败  :  " + e.getMessage());
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 获取SD下的应用目录
	 */
	public static String getExternalStoragePath() {
		StringBuilder sb = new StringBuilder();
		sb.append(Environment.getExternalStorageDirectory().getAbsolutePath());
		sb.append(File.separator);
		sb.append("Android");
		sb.append(File.separator);
		sb.append("data");
		sb.append(File.separator);
		sb.append(MyUtils.getContext().getPackageName());
		sb.append(File.separator);

		LogUtils.d("SD下的应用目录   :   " + sb.toString());
		return sb.toString();
	}

	/**
	 * 获取应用的cache目录
	 */
	public static String getCachePath() {
		StringBuilder sb = new StringBuilder();
		sb.append(File.separator);
		sb.append("data");
		sb.append(File.separator);
		sb.append("data");
		sb.append(File.separator);
		sb.append(MyUtils.getContext().getPackageName());
		sb.append(File.separator);
		sb.append("cache");
		sb.append(File.separator);
		return sb.toString();
	}

	/**
	 * 创建文件夹
	 */
	public static boolean createDirs(String dirPath) {
		if (!TextUtils.isEmpty(dirPath)) {
			File file = new File(dirPath);
			if (file.exists() && !file.isDirectory()) {
				if (file.delete()) {
					LogUtils.d("文件删除成功");
				} else {
					LogUtils.w("文件删除失败");
				}
			}
			if (!file.exists()) {
				if (file.mkdirs()) {
					LogUtils.d("创建" + dirPath + "   成功");
					return true;
				} else {
					LogUtils.d("创建" + dirPath + "   失败");
					return false;
				}
			} else {
				return true;
			}
		}
		return false;
	}

	/**
	 * 复制文件，可以选择是否删除源文件
	 */
	public static boolean copyFile(String srcPath, String destPath, boolean deleteSrc) {
		File srcFile = new File(srcPath);
		File destFile = new File(destPath);
		return copyFile(srcFile, destFile, deleteSrc);
	}

	/**
	 * 复制文件，可以选择是否删除源文件
	 */
	public static boolean copyFile(File srcFile, File destFile, boolean deleteSrc) {
		if (!srcFile.exists() || !srcFile.isFile()) {
			return false;
		}
		InputStream in = null;
		OutputStream out = null;
		try {
			in = new FileInputStream(srcFile);
			out = new FileOutputStream(destFile);
			byte[] buffer = new byte[1024];
			int i = -1;
			while ((i = in.read(buffer)) > 0) {
				out.write(buffer, 0, i);
				out.flush();
			}
			if (deleteSrc && srcFile.exists() && !srcFile.delete()) {
				return false;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		} finally {
			close(out, in);
		}
		return true;
	}

	public static String getString4Asset(String fileName) {
		BufferedReader tBufferedReader = null;
		try {
			InputStream is = MyUtils.getContext().getAssets().open(fileName);
			tBufferedReader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
			StringBuffer tStringBuffer = new StringBuffer();
			String sTempOneLine = null;
			while ((sTempOneLine = tBufferedReader.readLine()) != null) {
				tStringBuffer.append(sTempOneLine);
			}
			return tStringBuffer.toString();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			close(tBufferedReader);
		}
		return null;
	}

	/**
	 * 从输入流获取数据
	 *
	 * @param inputStream
	 * @return
	 * @throws Exception
	 */
	public static byte[] getByteFromStream(InputStream inputStream) {

		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		try {

			byte[] buffer = new byte[1024];
			int len = -1;
			while ((len = inputStream.read(buffer)) != -1) {
				outputStream.write(buffer, 0, len);
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			close(outputStream, inputStream);
		}
		return outputStream.toByteArray();
	}


	/**
	 * 关闭流
	 */
	public static void close(Closeable... io) {
		for (Closeable closeable : io) {
			if (closeable != null) {
				try {
					closeable.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * 关闭流
	 */
	public static void close(Cursor cursor) {
		if (cursor != null) {
			try {
				cursor.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * 判断文件是否可写
	 */
	public static boolean isWriteable(String path) {
		try {
			if (TextUtils.isEmpty(path)) {
				return false;
			}
			File f = new File(path);
			return f.exists() && f.canWrite();
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	/**
	 * 修改文件的权限,例如"777"等
	 */
	public static void chmod(String path, String mode) {
		try {
			String command = "chmod " + mode + " " + path;
			Runtime runtime = Runtime.getRuntime();
			runtime.exec(command);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 把数据写入文件
	 *
	 * @param is       数据流
	 * @param path     文件路径
	 * @param recreate 如果文件存在，是否需要删除重建
	 * @return 是否写入成功
	 */
	public static boolean writeFile(InputStream is, String path, boolean recreate) {
		boolean res = false;
		File f = new File(path);
		FileOutputStream fos = null;
		try {
			if (recreate && f.exists() && !f.delete()) {
				return res;
			}
			if (!f.exists() && null != is && createDirs(f.getParent())) {
				int count = -1;
				byte[] buffer = new byte[1024];
				fos = new FileOutputStream(f);
				while ((count = is.read(buffer)) != -1) {
					fos.write(buffer, 0, count);
				}
				res = true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			close(fos, is);
		}
		return res;
	}

	/**
	 * 把字符串数据写入文件
	 *
	 * @param content 需要写入的字符串
	 * @param path    文件路径名称
	 * @param append  是否以添加的模式写入
	 * @return 是否写入成功
	 */
	public static boolean writeFile(byte[] content, String path, boolean append) {
		boolean res = false;
		File f = new File(path);
		RandomAccessFile raf = null;
		try {
			if (f.exists()) {
				if (!append && (!f.delete() || !f.createNewFile())) {
					return res;
				}
			} else if (!f.createNewFile()) {
				return res;
			}
			if (f.canWrite()) {
				raf = new RandomAccessFile(f, "rw");
				raf.seek(raf.length());
				raf.write(content);
				res = true;
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			close(raf);
		}
		return res;
	}

	/**
	 * 把字符串数据写入文件
	 *
	 * @param content 需要写入的字符串
	 * @param path    文件路径名称
	 * @param append  是否以添加的模式写入
	 * @return 是否写入成功
	 */
	public static boolean writeFile(String content, String path, boolean append) {
		try {
			return writeFile(content.getBytes("UTF-8"), path, append);
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return false;
	}

	/**
	 * 把键值对写入文件
	 *
	 * @param filePath 文件路径
	 * @param key      键
	 * @param value    值
	 * @param comment  该键值对的注释
	 */
	public static void writeProperties(String filePath, String key, String value, String comment) {
		if (TextUtils.isEmpty(key) || TextUtils.isEmpty(filePath)) {
			return;
		}
		FileInputStream fis = null;
		FileOutputStream fos = null;
		File f = new File(filePath);
		try {
			if ((!f.exists() || !f.isFile()) && !f.createNewFile()) {
				return;
			}
			fis = new FileInputStream(f);
			Properties p = new Properties();
			p.load(fis);// 先读取文件，再把键值对追加到后面
			p.setProperty(key, value);
			fos = new FileOutputStream(f);
			p.store(fos, comment);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			close(fis, fos);
		}
	}

	/**
	 * 根据值读取
	 */
	public static String readProperties(String filePath, String key, String defaultValue) {
		if (TextUtils.isEmpty(key) || TextUtils.isEmpty(filePath)) {
			return null;
		}
		String value = null;
		FileInputStream fis = null;
		File f = new File(filePath);
		try {
			if ((!f.exists() || !f.isFile()) && !f.createNewFile()) {
				return null;
			}
			fis = new FileInputStream(f);
			Properties p = new Properties();
			p.load(fis);
			value = p.getProperty(key, defaultValue);
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			close(fis);
		}
		return value;
	}

	/**
	 * 把字符串键值对的map写入文件
	 */
	public static void writeMap(String filePath, Map<String, String> map, boolean append, String comment) {
		if (map == null || map.size() == 0 || TextUtils.isEmpty(filePath)) {
			return;
		}
		FileInputStream fis = null;
		FileOutputStream fos = null;
		File f = new File(filePath);
		try {
			if ((!f.exists() || !f.isFile()) && !f.createNewFile()) {
				return;
			}
			Properties p = new Properties();
			if (append) {
				fis = new FileInputStream(f);
				p.load(fis);// 先读取文件，再把键值对追加到后面
			}
			p.putAll(map);
			fos = new FileOutputStream(f);
			p.store(fos, comment);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			close(fis, fos);
		}
	}

	/**
	 * 把字符串键值对的文件读入map
	 */
	@SuppressWarnings({"rawtypes", "unchecked"})
	public static Map<String, String> readMap(String filePath, String defaultValue) {
		if (TextUtils.isEmpty(filePath)) {
			return null;
		}
		Map<String, String> map = null;
		FileInputStream fis = null;
		File f = new File(filePath);
		try {
			if ((!f.exists() || !f.isFile()) && !f.createNewFile()) {
				return map;
			}
			fis = new FileInputStream(f);
			Properties p = new Properties();
			p.load(fis);
			map = new HashMap<String, String>((Map) p);// 因为properties继承了map，所以直接通过p来构造一个map
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			close(fis);
		}
		return map;
	}

	/**
	 * 改名
	 */
	public static boolean copy(String src, String des, boolean delete) {
		File file = new File(src);
		if (!file.exists()) {
			return false;
		}
		File desFile = new File(des);
		FileInputStream in = null;
		FileOutputStream out = null;
		try {
			in = new FileInputStream(file);
			out = new FileOutputStream(desFile);
			byte[] buffer = new byte[1024];
			int count = -1;
			while ((count = in.read(buffer)) != -1) {
				out.write(buffer, 0, count);
				out.flush();
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		} finally {
			close(in, out);
		}
		if (delete) {
			if (file.delete()) {
				LogUtils.d("文件删除成功");
			} else {
				LogUtils.w("文件删除失败");
			}
		}
		return true;
	}

	/**
	 * 复制文件，支持断点续传
	 *
	 * @param in
	 * @param destFile
	 * @return
	 * @throws IOException
	 */
	public static boolean copyFile(InputStream in, File destFile)
			throws IOException {
		if (in == null) {
			return false;
		}
		RandomAccessFile out = null;
		try {
			out = new RandomAccessFile(destFile, "rw");
			out.seek(destFile.length());
			byte[] buffer = new byte[1024];
			int i = -1;
			while ((i = in.read(buffer)) > 0) {
				out.write(buffer, 0, i);
			}
		} finally {
			close(out, in);
		}
		return true;
	}
}
